{-# LANGUAGE OverloadedStrings #-}
module Server where

import Data.Text(Text, pack, unpack)
import Data.Maybe(fromMaybe)
import Control.Monad(join)
import Data.String
import Data.List(intercalate)
import Data.ByteString(ByteString) -- hiding (pack, unpack, intercalate)

import Network.Wai
import Network.HTTP.Types (status200)
import Network.Wai.Handler.Warp
import Network.HTTP.Types.URI
import Network.HTTP.Types.Method

import qualified Text.Blaze.Html5 as H
import Text.Blaze((!), stringValue)
import qualified Text.Blaze.Html5.Attributes as A
import Text.Blaze.Html.Renderer.Utf8

import qualified ExamplesDraw as Examples

mainPage :: String-> H.Html -> H.Html
mainPage name gfx = 
  H.docTypeHtml $ do
    H.head $ do
      H.title "Graphics Examples"
    H.body $ do
      H.h1 $ H.text "Example"
      H.br
      gfx
      H.br
      H.form  $ do
      	H.select ! A.name "example" $ do
	  H.option ! A.value "ex1" $ H.text "Example 1"
	  H.option ! A.value "ex2" $ H.text "Example 2"
	  H.option ! A.value "ex4" $ H.text "Example 4"
	  H.option ! A.value "ex5" $ H.text "Example 5"
	  H.option ! A.value "ex6" $ H.text "Example 6"
	H.br
        H.input ! A.type_ "submit" ! A.value "Show me!"
	
page :: QueryText -> H.Html
page qt
  | pack "example" `elem` (map fst qt) =
     case (fmap unpack $ join $ lookup (pack "example") qt) of
       Just "ex1" -> mainPage "Example 1" Examples.ex1
       Just "ex2" -> mainPage "Example 2" Examples.ex2
       Just "ex4" -> mainPage "Example 4" Examples.ex4
       Just "ex5" -> mainPage "Example 5" Examples.ex5
       Just "ex6" -> mainPage "Example 6" Examples.ex6
       _ -> mainPage "Welcome" $ H.div "Please select example below."
  | otherwise =
    mainPage "Welcome" $ H.div "Please select example below."   

app :: Application
app req respond = 
    let p = page (queryToQueryText $ queryString req)
    in  respond $ responseBuilder status200 [] $ renderHtmlBuilder p

-- Starte die Webanwendung auf Port 8000
main =run 8000 $ app
